import 'dart:io';

import 'package:path/path.dart';

Future<void> starter(List<String> args) async {
  List<String> params = args.toList();

  String org = '';
  String android = '';
  String ios = '';
  String name = '';
  String template = 'provider';

  List<String> fcargs = [];

  if (params.contains('--p')) {
    int idx = params.indexOf('--p');
    name = params[idx + 1];
    fcargs.add('--project-name');
    fcargs.add(name);
    params.removeAt(idx);
    params.removeAt(idx);
  }

  if (params.contains('--org')) {
    int idx = params.indexOf('--org');
    org = params[idx + 1];
    fcargs.add('--org');
    fcargs.add(org);
    params.removeAt(idx);
    params.removeAt(idx);
  }

  if (params.contains('--a')) {
    int idx = params.indexOf('--a');
    android = params[idx + 1];
    fcargs.add('--android-language');
    fcargs.add(android);
    params.removeAt(idx);
    params.removeAt(idx);
  }

  if (params.contains('--i')) {
    int idx = params.indexOf('--i');
    ios = params[idx + 1];
    fcargs.add('--ios-language');
    fcargs.add(ios);
    params.removeAt(idx);
    params.removeAt(idx);
  }

  if (params.contains('--t')) {
    int idx = params.indexOf('--t');
    template = params[idx + 1].toLowerCase();
    params.removeAt(idx);
    params.removeAt(idx);
  }

  String project = params.first;
  fcargs.add(project);

  Process process = await Process.start(
    'flutter',
    ['create', ...fcargs],
    runInShell: true,
    mode: ProcessStartMode.inheritStdio,
  );

  process.exitCode.then((_) {
    print('');
    print('Installing dependencies...');
    print('');

    _install(project, 'intl');
    _install(project, 'http');
    _install(project, 'uuid');

    switch (template) {
      case 'bloc':
        _install(project, 'equatable');
        _install(project, 'rxdart');
        _install(project, 'bloc');
        _install(project, 'flutter_bloc');
        break;
      case 'cubit':
        _install(project, 'equatable');
        _install(project, 'bloc');
        _install(project, 'flutter_bloc');
        break;
      case 'get':
      case 'getx':
        _install(project, 'equatable');
        _install(project, 'get');
        break;
      case 'mobx':
        _install(project, 'mobx');
        _install(project, 'mobx_codegen');
        _install(project, 'flutter_mobx');
        break;
      case 'riverpod':
        _install(project, 'equatable');
        _install(project, 'flutter_riverpod');
        break;
      case 'provider':
      default:
        _install(project, 'provider');
        break;
    }

    _templateAll(project);

    switch (template) {
      case 'bloc':
        _templateBloc(project);
        break;
      case 'cubit':
        _templateCubit(project);
        break;
      case 'get':
      case 'getx':
        _templateGetx(project);
        break;
      case 'mobx':
        _templateMobx(project);
        break;
      case 'riverpod':
        _templateRiverpod(project);
        break;
      case 'provider':
      default:
        _templateProvider(project);
        break;
    }

    exit(0);
  });
}

void _install(String project, String package) {
  print('install $package');
  Process.runSync('cd $project && pub add $package', [], runInShell: true);
}

void _templateAll(String project) {
  print('');
  print('Creating assets...');
  print('');

  print('  /assets');
  Directory('$project/assets').createSync();

  print('  /assets/fonts');
  Directory('$project/assets/fonts').createSync();

  print('  /assets/icons');
  Directory('$project/assets/icons').createSync();

  print('  /assets/html');
  Directory('$project/assets/html').createSync();

  print('  /assets/images');
  Directory('$project/assets/images').createSync();

  print('  /assets/logo');
  Directory('$project/assets/logo').createSync();

  print('');
  print('Creating template ${absolute(project)}...');

  Directory('$project/lib/api').createSync();
  Directory('$project/lib/config').createSync();
  Directory('$project/lib/models').createSync();
  Directory('$project/lib/modules').createSync();
  Directory('$project/lib/routes').createSync();
  Directory('$project/lib/screens').createSync();
  Directory('$project/lib/utils').createSync();
  Directory('$project/lib/widgets').createSync();

  _file(project, '/lib/app.dart');
  _main(project);
  _app(project);

  _file(project, '/lib/api/api.dart');
  _file(project, '/lib/api/api_constant.dart');
  _file(project, '/lib/api/api_handler.dart');
  _file(project, '/lib/api/api_exception.dart');
  _api(project);

  _file(project, '/lib/config/config.dart');
  _file(project, '/lib/config/assets.dart');
  _file(project, '/lib/config/colors.dart');
  _file(project, '/lib/config/fonts.dart');
  _file(project, '/lib/config/strings.dart');
  _file(project, '/lib/config/styles.dart');
  _file(project, '/lib/config/themes.dart');
  _config(project);

  _file(project, '/lib/models/user.dart');
  _models(project);

  _file(project, '/lib/routes/routes.dart');
  _routes(project);

  _file(project, '/lib/screens/screens.dart');
  _file(project, '/lib/screens/home.dart');
  _file(project, '/lib/screens/splash.dart');
  _screens(project);

  _file(project, '/lib/utils/theme.dart');

  _file(project, '/lib/widgets/logo.dart');
  _widgets(project);
}

void _templateProvider(String project) {
  _file(project, '/lib/modules/auth.dart');
  _file(project, '/lib/modules/home.dart');
}

void _templateBloc(String project) {
  Directory('$project/lib/modules/auth').createSync();
  Directory('$project/lib/modules/home').createSync();

  _file(project, '/lib/modules/auth/auth.dart');
  _file(project, '/lib/modules/auth/auth_event.dart');
  _file(project, '/lib/modules/auth/auth_state.dart');
  _file(project, '/lib/modules/home/home.dart');
  _file(project, '/lib/modules/home/home_event.dart');
  _file(project, '/lib/modules/home/home_state.dart');
}

void _templateCubit(String project) {
  Directory('$project/lib/modules/auth').createSync();
  Directory('$project/lib/modules/home').createSync();

  _file(project, '/lib/modules/auth/auth.dart');
  _file(project, '/lib/modules/auth/auth_state.dart');
  _file(project, '/lib/modules/home/home.dart');
  _file(project, '/lib/modules/home/home_state.dart');
}

void _templateGetx(String project) {
  Directory('$project/lib/modules/auth').createSync();
  Directory('$project/lib/modules/home').createSync();

  _file(project, '/lib/modules/auth/auth.dart');
  _file(project, '/lib/modules/auth/auth_bindings.dart');
  _file(project, '/lib/modules/home/home.dart');
  _file(project, '/lib/modules/home/home_bindings.dart');
}

void _templateMobx(String project) {
  _file(project, '/lib/modules/auth.dart');
  _file(project, '/lib/modules/home.dart');
}

void _templateRiverpod(String project) {
  Directory('$project/lib/modules/auth').createSync();
  Directory('$project/lib/modules/home').createSync();

  _file(project, '/lib/modules/auth/auth.dart');
  _file(project, '/lib/modules/auth/auth_state.dart');
  _file(project, '/lib/modules/home/home.dart');
  _file(project, '/lib/modules/home/home_state.dart');
}

void _file(String project, String path) {
  print('  $path');
  File('$project$path').createSync();
}

void _main(String project) {
  File('$project/lib/main.dart').writeAsStringSync('''
import 'package:flutter/material.dart';
import 'package:$project/app.dart';

void main() {
  WidgetsFlutterBinding.ensureInitialized();
  runApp(App());
}
  ''');
}

void _app(String project) {
  File('$project/lib/app.dart').writeAsStringSync('''
import 'package:flutter/material.dart';
import 'package:$project/config/config.dart';
import 'package:$project/screens/home.dart';

class App extends StatelessWidget {
  const App({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: appName,
      theme: appTheme,
      home: Home(),
    );
  }
}
  ''');
}

void _api(String project) {
  File('$project/lib/api/api.dart').writeAsStringSync('''
import 'dart:convert';
import 'dart:io';
import 'package:http/http.dart' as http;

part 'api_constant.dart';
part 'api_exception.dart';
part 'api_handler.dart';

class Api {
  static final ApiHandler _handler = ApiHandler();

  static Future users(int page) async {
    return await _handler.get(userUrl);
  }
}
  ''');

  File('$project/lib/api/api_constant.dart').writeAsStringSync('''
part of 'api.dart';

const String baseUrl = 'https://gorest.co.in/public/v1';
const String userUrl = '\$baseUrl/users';
  ''');

  File('$project/lib/api/api_handler.dart').writeAsStringSync('''
part of 'api.dart';

abstract class ApiHandler {
  Future get(String url);
  Future post(String url, dynamic body);
  Future put(String url, dynamic body);
  Future delete(String url);
  
  factory ApiHandler() => _ApiHandlerImpl();
}

class _ApiHandlerImpl implements ApiHandler {
  @override
  Future get(String url) async {
    try {
      final response = await http.get(Uri.parse(url));
      return _process(response);
    } on SocketException {
      throw FetchDataException('No Internet connection');
    }
  }

  @override
  Future post(String url, body) async {
    try {
      final response = await http.post(Uri.parse(url), body: body);
      return _process(response);
    } on SocketException {
      throw FetchDataException('No Internet connection');
    }
  }

  @override
  Future put(String url, body) async {
    try {
      final response = await http.put(Uri.parse(url), body: body);
      return _process(response);
    } on SocketException {
      throw FetchDataException('No Internet connection');
    }
  }

  @override
  Future delete(String url) async {
    try {
      final response = await http.delete(Uri.parse(url));
      return _process(response);
    } on SocketException {
      throw FetchDataException('No Internet connection');
    }
  }

  dynamic _process(http.Response response) {
    switch (response.statusCode) {
      case 200:
      case 400:
      case 401:
      case 403:
      case 404:
        return json.decode(response.body.toString());
      case 500:
      default:
        return FetchDataException(
          'Error occured while Communication with Server with StatusCode : \${response.statusCode}',
        );
    }
  }
}
  ''');

  File('$project/lib/api/api_exception.dart').writeAsStringSync('''
part of 'api.dart';

class AppException implements Exception {
  final String? _message;
  final String? _prefix;

  AppException([this._message, this._prefix]);

  @override
  String toString() {
    return '\$_prefix\$_message';
  }
}

class FetchDataException extends AppException {
  FetchDataException([message])
      : super(message, 'Error During Communication: ');
}

class BadRequestException extends AppException {
  BadRequestException([message]) : super(message, 'Invalid Request: ');
}

class UnauthorisedException extends AppException {
  UnauthorisedException([message]) : super(message, 'Unauthorised: ');
}
  ''');
}

void _config(String project) {
  File('$project/lib/config/config.dart').writeAsStringSync('''
export 'assets.dart';
export 'colors.dart';
export 'fonts.dart';
export 'strings.dart';
export 'styles.dart';
export 'themes.dart';
  ''');

  File('$project/lib/config/assets.dart').writeAsStringSync('''

  ''');

  File('$project/lib/config/colors.dart').writeAsStringSync('''
import 'package:flutter/material.dart';

MaterialColor primarySwatch = Colors.blue;
Color primaryColor = Colors.blue;
  ''');

  File('$project/lib/config/fonts.dart').writeAsStringSync('''

  ''');

  File('$project/lib/config/strings.dart').writeAsStringSync('''
String appName = 'Flutter App';
String appBarTitle = 'Flutter App';
String search = 'Search...';
  ''');

  File('$project/lib/config/themes.dart').writeAsStringSync('''
import 'package:flutter/material.dart';
import 'package:$project/config/colors.dart';

ThemeData appTheme = ThemeData(
  primarySwatch: primarySwatch,
  primaryColor: primaryColor,
);

  ''');
}

void _models(String project) {
  File('$project/lib/models/user.dart').writeAsStringSync('''
class User {
  final int id;
  final String name;
  final String email;
  final String? gender;
  final String? status;

  bool get isActive => status == 'active';

  User({
    required this.id,
    required this.name,
    required this.email,
    this.gender,
    this.status,
  });

  User.fromJson(Map<String, dynamic> json)
      : id = int.tryParse(json['id']?.toString() ?? '') ?? 0,
        name = json['name']?.toString() ?? '',
        email = json['email']?.toString() ?? '',
        gender = json['gender']?.toString(),
        status = json['status']?.toString();

  Map<String, dynamic> toJson() => {
        'id': id,
        'name': name,
        'email': email,
        'gender': gender,
        'status': status,
      };
}
  ''');
}

void _routes(String project) {
  File('$project/lib/routes/routes.dart').writeAsStringSync('''

  ''');
}

void _screens(String project) {
  File('$project/lib/screens/screens.dart').writeAsStringSync('''
export 'home.dart';
export 'splash.dart';
  ''');
}

void _utils(String project) {
  File('$project/lib/utils/theme.dart').writeAsStringSync('''

  ''');
}

void _widgets(String project) {
  File('$project/lib/widgets/logo.dart').writeAsStringSync('''

  ''');
}
